/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "IDrawable.hpp"
#include "allegro5/allegro.h"
#include "allegro5/allegro_primitives.h"
#include <iostream>

void IDrawable::UseNewTransformTo(int x,int y){  
    ALLEGRO_TRANSFORM temp_transform;
    al_identity_transform(&temp_transform);
    al_translate_transform(&temp_transform, x, y);
    al_use_transform(&temp_transform);
}

IDrawable::IDrawable(){
    parent = nullptr;
    _last_drawn_height = -1;
    _last_drawn_width = -1;
    _needs_redrawing = true;
    _cache_bitmap = al_create_bitmap(0,0);
    
    _background_color = al_map_rgba(0,0,0,0); //fully transparent background by default
    _margin = 0; // by default: no margin
}

IDrawable::~IDrawable(){
    al_destroy_bitmap(_cache_bitmap);
}


void IDrawable::_ResizeCacheBitmap(int w, int h){
    al_destroy_bitmap(_cache_bitmap);
    _cache_bitmap = al_create_bitmap(w,h);
}

void IDrawable::SetBackgroundColor(ALLEGRO_COLOR bg){
    _background_color = bg;
}
void IDrawable::RemoveBackgroundColor(){
    _background_color = al_map_rgba(0,0,0,0);
}

void IDrawable::SetMargin(int margin)
{
     _margin = margin;
}

void IDrawable::Draw(SDrawArea area){
    if(_needs_redrawing || area.w != _last_drawn_width || area.h != _last_drawn_height){
        // Somethings changed, this drawable's buffer needs to be redrawn
        
        if(area.w != _last_drawn_width || area.h != _last_drawn_height) _ResizeCacheBitmap(area.w, area.h);
        
        ALLEGRO_BITMAP* last_bitmap = al_get_target_bitmap();
        al_set_target_bitmap(_cache_bitmap);
        al_clear_to_color(_background_color);
        
        UseNewTransformTo(_margin,_margin);
        _Redraw(area.w-2*_margin, area.h-2*_margin);
        
        al_set_target_bitmap(last_bitmap);
        
        _needs_redrawing = false;
        _last_drawn_height = area.h;
        _last_drawn_width = area.w;
    }
    
    // Draw this widget's cached version
    al_draw_bitmap(_cache_bitmap, area.x, area.y, 0);
    
}

void IDrawable::SetNeedsRedrawing(){
    _needs_redrawing = true;
    if(parent != nullptr) parent->SetNeedsRedrawing(); 
}

int IDrawable::GetMinimalWidth ()
{
    int max = _GetInnerMinimalWidth() + 2*_margin;
    if(custom_min_width > max) max = custom_min_width;
    return max;
}
int IDrawable::GetMinimalHeight()
{
    int max = _GetInnerMinimalHeight() + 2*_margin;
    if(custom_min_height > max) max = custom_min_height;
    return max;
}

void IDrawable::SetCustomMinimalHeight(int m){ custom_min_height = m;}
void IDrawable::SetCustomMinimalWidth (int m){ custom_min_width  = m;}

void IDrawable::OnClicked(SCoordinates p){
    if(p.x < 0 || p.y < 0 || p.x > _last_drawn_width || p.y > _last_drawn_height)
        return;

    if(p.x <= _margin || p.y <= _margin || p.x > _last_drawn_width-_margin || p.y > _last_drawn_height-_margin){
	event_margin_clicked.Happen();
    }else{
	event_clicked.Happen(SCoordinates(p.x-_margin,p.y-_margin));
    }
}

void IDrawable::OnMouseIn(){
    pointed = true;
}
void IDrawable::OnMouseOut(){
    pointed = false;
    if(inner_pointed){
	inner_pointed = false;
	event_mouse_out.Happen();
    }
}
void IDrawable::OnMouseMovedOver(SCoordinates p){
    if(p.x < 0 || p.y < 0 || p.x > _last_drawn_width || p.y > _last_drawn_height)
        return;
    if(p.x <= _margin || p.y <= _margin || p.x > _last_drawn_width-_margin || p.y > _last_drawn_height-_margin){
	// mouse is outside the inner part of the widget
	if(inner_pointed){
	    event_mouse_out.Happen();
	    inner_pointed = false;
	}
    }else{
	// mouse is inside the inner part of the widget
	if(!inner_pointed){
	    event_mouse_in.Happen();
	    inner_pointed = true;
	}
	event_mouse_over.Happen(SCoordinates(p.x - _margin, p.y - _margin));
    }
    
}

void IDrawable::OnKeypress(SKeypressData k){
    event_keypress.Happen(k);
}
