#ifndef __IDRAWABLE_HPP__
#define __IDRAWABLE_HPP__

/* This file is part of Millennium Duel.
Copyright 2013 Rafał Cieślak and the Millennium Developers Team

Millennium Duel is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

Millennium Duel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License
along with Millennium Duel; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA  */

#include "CEvent.hpp"
#include <allegro5/allegro.h>

struct SDrawArea{
    SDrawArea(int _x, int _y, int _w, int _h) : x(_x), y(_y), w(_w), h(_h) {};
    int x;
    int y;
    int w;
    int h;
};

struct SCoordinates{
    int x;
    int y;
    SCoordinates(int _x, int _y) : x(_x), y(_y) {}
    SCoordinates() : x(0), y(0) {}
};
struct SKeypressData{
    SKeypressData(int keycode, unsigned modifiers, int unicode) : allegro_keycode(keycode), allegro_modifiers(modifiers), allegro_unicode(unicode) {}
    int allegro_keycode;
    unsigned allegro_modifiers;
    int allegro_unicode;
};


class IDrawable{
public:
    IDrawable();
    virtual ~IDrawable();

    /* Used to set the color used as the background for this widget.
     * It will be visible in all areas that are allocated by this widget,
     * but not drawn (for example, margin, spacing between items in a Box
     * etc. ) */
    void SetBackgroundColor(ALLEGRO_COLOR);
    /* Used to restore the default, transparent background */
    void RemoveBackgroundColor();

    void SetMargin(int margin); // - set widget's margin size

    /* This method can be called to draw this widget's graphical contents
     * onto the current surface, in the place specified by the argument
     * (coordinates + size). If possible, this method will avoid rendering
     * the widget and will copy a cached bitmap instead, to maximize efficency.
     * Also, it takes care of widget's margin.
     * This method should be never overwritten by any widgets. */
    void Draw(SDrawArea area);
    
    /* This method should be called on widget when clicked.
     * It should be never overwritten by any widgets. */
    void OnClicked(SCoordinates p); //click coordinates are relative to widget position

    void OnMouseMovedOver(SCoordinates p);
    void OnMouseIn();
    void OnMouseOut();
    
    void OnKeypress(SKeypressData k);
    
    /* These events should NEVER be triggered by anyone outside this class, including
     * all custom widgets! If you wish to notify a widget about such event, use
     * appropriate On* methods above */
    CEvent<SCoordinates> event_clicked;
    CEvent<> event_margin_clicked;
    CEvent<SCoordinates> event_mouse_over;
    CEvent<> event_mouse_in;
    CEvent<> event_mouse_out;
    CEvent<SKeypressData> event_keypress;
    
    /* This variable is used to remember whether the mouse is currently over this
     * widget */
    bool pointed = false;
    /* And this one remembers whether the mouse is currently over the inner part
     * of the widget (not margin) */
    bool inner_pointed = false;

    /* These two methods can be used to calculate widget's total size. They
     * make use of GetInnerMinimalWidth and GetInnerMinimalHeight
     * They should be never overwritten by any widgets. */
    int GetMinimalWidth ();
    int GetMinimalHeight();
    
    /* Use these methods to specify a custom requirements for minimal dimentions
     * of a widget. The widget will be then guaranteed to never allocate less
     * space then specified. The wiget may be larger, though, both in case
     * it's own minimal size is greater than the provided one, or in case the
     * widget is given more space than needed and it has to expand. Default: 0 */
    void SetCustomMinimalWidth(int m);
    void SetCustomMinimalHeight(int m);
    
    /* A helpful macro, used by probably every drawable in its draw() func */
    static void UseNewTransformTo(int x, int y);
    
    /* A convenience function that ensures this widget will be re-rendered next
     * time it gets drawn, as well as its parent, and the parent's parent, and
     * so on, to propagate the change.
     * When implmenting a custom widget, you should call this whenever its
     * internal state changes in a way that modifies the widgets appearance
     * (e.g. when button's text is changed). */
    void SetNeedsRedrawing();
    
    /* The widget that contains this one.
     * nullptr, if this widget has no parent */
    IDrawable* parent;
    
    bool _needs_redrawing;
private:
    ALLEGRO_COLOR _background_color; 
    int _margin;
    
    /* The redraw(...) method is used to re-render this widget's contents.
     * It is used by draw(...) in case the cached bitmap requires updating.
     * This is the method you should override when creating a custom widget.
     * The caller takes care of proper target bitmap, transformation, the
     * background and margin size, so you do not need to worry about these
     * - just draw your widget from (0,0) to (w,h) (ignoring background and
     * margin).
     * Looking at other widgets' redraw(...) may be helpful when writing a
     * new widget */ 
    virtual void _Redraw(int w, int h) = 0;
    
    /* These methods should calculate widget's interior's size (excluding the
     * margin).
     * These are the methods you should override when creating a custom widget. */
    virtual int _GetInnerMinimalWidth () = 0;
    virtual int _GetInnerMinimalHeight() = 0;
    
    int custom_min_width = 0;
    int custom_min_height = 0;
    
    /* The cached appearance of this widget. */
    ALLEGRO_BITMAP* _cache_bitmap;
    /* A convenience method that creates a new bitmap of a new size. */
    void _ResizeCacheBitmap(int w, int h);
    
    /* The widget size (as it was drawn the last time) is stored to detect when
     * a widget is resized and needs redrawing. */
    int _last_drawn_width;
    int _last_drawn_height;
};

#endif //__IDRAWABLE_HPP__
